import math
import numpy
import re


### Vector math support

def multiplyvec(vec1,vec2):
    if len(vec1)!=len(vec2):
        print 'Vector dimensions are not consistent!'
        return []
    return map(lambda x,y:x*y,vec1,vec2)

def subvec(vec1,vec2):
    if len(vec1)!=len(vec2):
        print 'Vector dimensions are not consistent!'
        return []
        
    return map(lambda x,y: x-y, vec1,vec2)

def addvec(vec1,vec2):
    if len(vec1)!=len(vec2):
        print 'Vector dimensions are not consistent!'
        return []        
    return map(lambda x,y: x+y, vec1,vec2)

def averagevecarray(vecarray):
    if len(vecarray)==0:
        print 'Zero length vector!'
        return []
    
    temp=reduce(addvec,vecarray)
    averagevec=[float(x/len(vecarray)) for x in temp]
    return averagevec

def averagevec(vec):
    if len(vec)==0:
        print 'Zero length vector!'
        return

    length=len(vec)
    average=0
    for i in range(length):
        average+= float(vec[i])/length
    
    return average
    
def medium(vec):
    if len(vec)==0:
        print 'Zero length vector!'
        return
    
    temp=[vec[i] for i in range(len(vec))]
    temp.sort()
    return temp[len(vec)/2]

def min_medium_max(vec):
    if len(vec)==0:
        print 'Zero length vector!'
        return
    
    temp=[vec[i] for i in range(len(vec))]
    temp.sort()
    return [temp[0],temp[len(vec)/2],temp[len(vec)-1]]

def vec2str(vec,sep='\t'):
    if len(vec)==0:
        print 'Zero length vector!'
        return

    string=''

    for i in range(len(vec)-1):
        string=string+str(vec[i])+sep

    string=string+str(vec[len(vec)-1])
    return string

#08/10/2007
def norm(vec):

    if len(vec) == 0:
        return 0.

    sum = 0
    for i in vec:
        sum += i * i;
        
    return math.sqrt(sum)

def dot(vec1, vec2):

    num = len(vec1)
    if num != len(vec2):
        return 0.

    result = 0.;
    for i in range(num):
        result += vec1[i] * vec2[i];
        
    return result

import copy
def scale(scale, vec):
    
    rvec = copy.copy(vec[:])
    for i in range(len(vec)):
        rvec[i] =  scale * vec[i]        
    return rvec

def unit(vec):
    vnorm = norm(vec)
    if (vnorm < 0.00000000001):
        vnorm = 0.00000001        
    return scale(1.0/vnorm, vec)

    
def divvec(vec1,vec2):
    if len(vec1)!=len(vec2):
        print 'Vector dimensions are not consistent!'
        return []
    return map(lambda x,y:x/y,vec1,vec2)

def vecabs(vec):	
	if len(vec)==0:
		return [];	
	tmp = vec[:]
	print tmp
	for i in range(len(tmp)):
  		tmp[i] = abs(tmp[i])
 		
	return tmp    

def sum(vec):
	if len(vec)==0:
		return;
	sum = 0.
	for i in range(len(vec)):
		sum += vec[i]
	return sum
		
    
def vecstd(vec):
	if len(vec)==0:
		return [];
	meanvec = averagevec(vec)
	tmp = vec[:]
	for i in range(len(tmp)):
		tmp[i] -= meanvec
		
	tmp2 = multiplyvec(tmp, tmp)	
	meantmp2 = averagevec(tmp2)
	return math.sqrt(meantmp2)

def MaxVector(vectorSet):
     vectorNum=len(vectorSet)
     vectorDim=len(vectorSet[0])
     
     maxVector=[[]]*3
     for ii in range(vectorDim):
          coord=[]
          for i in range(vectorNum):
               coord.append(vectorSet[i][ii])          
          maxVector[ii]=max(coord)

     return maxVector

def MinVector(vectorSet):
     vectorNum=len(vectorSet)
     vectorDim=len(vectorSet[0])
     
     minVector=[[]]*3
     for ii in range(vectorDim):
          coord=[]
          for i in range(vectorNum):
               coord.append(vectorSet[i][ii])          
          minVector[ii]=min(coord)

     return minVector

def crossproduct(vec1,vec2):
    if len(vec1)!=3 or len(vec2)!=3:
        print 'This function only support cross product for 3D vector!'
        return

    resultVec=[0]*3

    resultVec[0]=vec1[1]*vec2[2]-vec1[2]*vec2[1]
    resultVec[1]=vec1[2]*vec2[0]-vec1[0]*vec2[2]
    resultVec[2]=vec1[0]*vec2[1]-vec1[1]*vec2[0]
    
    return resultVec

import math
def abs(vec):
    temp=[math.fabs(vec[i]) for i in range(len(vec))]
    return temp
    
def distvec(vect1, vect2):
    return norm(subvec(vect1, vect2))
    
### Transformation tool support

def EstimateRigidTransformation(srcPointSet,destPointSet):
    #Check if srcPointSet and destPointSet have the same number of points
    if len(srcPointSet)!=len(destPointSet):
        print 'Inconsistent sizes between two point sets!'
        return [[],[]]

    #Guarentee there are more than 3 points
    if len(srcPointSet)<3:
        print 'Not enough point to estimate rigid transformation'
        return [[],[]]

    pointNum=len(srcPointSet)

    #Calculate the center of each point set
    avgSrc=[0]*3
    avgDest=[0]*3
    
    for i in range(pointNum):
        avgSrc=addvec(avgSrc,srcPointSet[i])
        avgDest=addvec(avgDest,destPointSet[i])

    avgSrc=scale(1.0/float(pointNum),avgSrc)
    avgDest=scale(1.0/float(pointNum),avgDest)
    
    #print 'avgSrc=',avgSrc
    #print 'avgDest=',avgDest

    #Centralize the pointset
    cSrcPoint=[[]]*pointNum
    cDestPoint=[[]]*pointNum

    for i in range(pointNum):
        cSrcPoint[i]=subvec(srcPointSet[i],avgSrc)
        cDestPoint[i]=subvec(destPointSet[i],avgDest)

    #print 'cSrcPoint=',cSrcPoint
    #print 'cDestPoint=',cDestPoint
    #SVD decomposition
    sum=numpy.zeros((3,3))
    for i in range(pointNum):
        sum += numpy.dot(numpy.transpose([cSrcPoint[i]]),[cDestPoint[i]])
    
    u,s,vh=numpy.linalg.svd(sum)
    
    #Calculate rotation matrix
    v=numpy.transpose(vh)    
    
#     print 'u=',u
#     print 's=',s
#     print 'v=',v
#     
    rotation=numpy.dot(v,numpy.transpose(u))

    #Check reflection status
    minSig=s[0]
    for ii in range(3):
        if s[ii]<minSig:
            minSig=s[ii]
    
    # Only checking for determinant condition
    if numpy.linalg.det(rotation)<0: #and math.fabs(minSig)<0.0001:
        v[:,2]=-v[:,2]
        rotation=numpy.dot(v,numpy.transpose(u))

    translation=numpy.transpose([avgDest])-numpy.dot(rotation,numpy.transpose([avgSrc]))
    return [rotation,translation]

def EstimateRigidTransformationGeneral(srcPointSet,destPointSet,dimension):
    #Check if srcPointSet and destPointSet have the same number of points
    if len(srcPointSet)!=len(destPointSet):
        print 'Inconsistent sizes between two point sets!'
        return [[],[]]

    #Guarentee there are more than 3 points
    if len(srcPointSet)<3:
        print 'Not enough point to estimate rigid transformation'
        return [[],[]]

    pointNum=len(srcPointSet)

    #Calculate the center of each point set
    avgSrc=[0]*dimension
    avgDest=[0]*dimension
    
    for i in range(pointNum):
        avgSrc=addvec(avgSrc,srcPointSet[i])
        avgDest=addvec(avgDest,destPointSet[i])

    avgSrc=scale(1.0/float(pointNum),avgSrc)
    avgDest=scale(1.0/float(pointNum),avgDest)
    
    #print 'avgSrc=',avgSrc
    #print 'avgDest=',avgDest

    #Centralize the pointset
    cSrcPoint=[[]]*pointNum
    cDestPoint=[[]]*pointNum

    for i in range(pointNum):
        cSrcPoint[i]=subvec(srcPointSet[i],avgSrc)
        cDestPoint[i]=subvec(destPointSet[i],avgDest)

    #print 'cSrcPoint=',cSrcPoint
    #print 'cDestPoint=',cDestPoint
    #SVD decomposition
    sum=numpy.zeros((dimension,dimension))
    for i in range(pointNum):
        sum += numpy.dot(numpy.transpose([cSrcPoint[i]]),[cDestPoint[i]])
    
    u,s,vh=numpy.linalg.svd(sum)
    
    #Calculate rotation matrix
    v=numpy.transpose(vh)    
    
#     print 'u=',u
#     print 's=',s
#     print 'v=',v
#     
    rotation=numpy.dot(v,numpy.transpose(u))

    #Check reflection status
    minSig=s[0]
    for ii in range(dimension):
        if s[ii]<minSig:
            minSig=s[ii]
    
    # Only checking for determinant condition
    if numpy.linalg.det(rotation)<0: #and math.fabs(minSig)<0.0001:
        v[:,dimension-1]=-v[:,dimension-1]
        rotation=numpy.dot(v,numpy.transpose(u))

    translation=numpy.transpose([avgDest])-numpy.dot(rotation,numpy.transpose([avgSrc]))
    return [rotation,translation]

def ApplyRigidTransformToPoint(point,rotation,translation):
    pp=numpy.transpose([point])
    tpp=numpy.dot(rotation,pp)+translation
    transformedPoint=[0]*3
    for ii in range(3):
        transformedPoint[ii]=tpp[ii][0]
    return transformedPoint
    

    
def EstimateSimilarityTransformation(srcPointSet,destPointSet):
    if len(srcPointSet)!=len(destPointSet):
        print 'Inconsistent sizes between two point sets!'
        return [[]]*len(destPointSet)

    if len(srcPointSet)<3:
        print 'Not enough point to estimate similarity transformation'
        return [[]]*len(destPointSet)
    
    pointNum=len(srcPointSet)
    
    #Compute mean
    avgSrc=[0]*3
    avgDest=[0]*3
    
    for i in range(pointNum):
        avgSrc=addvec(avgSrc,srcPointSet[i])
        avgDest=addvec(avgDest,destPointSet[i])
    avgSrc=scale(1.0/float(pointNum),avgSrc)
    avgDest=scale(1.0/float(pointNum),avgDest)
    
    # compute variance of the source points
    srcTranspose = numpy.transpose(srcPointSet)
    sigma_source = 0
    for j in range(3):
        sigma_source += numpy.var(srcTranspose[j])
    
    # get translated source and destination points
    cSrcPoint=[[]]*pointNum
    cDestPoint=[[]]*pointNum
    for i in range(pointNum):
        cSrcPoint[i]=subvec(srcPointSet[i],avgSrc)
        cDestPoint[i]=subvec(destPointSet[i],avgDest)
    
    # compute covariance matrix between source and destination points
    covariance = []
    cDestTranspose = numpy.transpose(cDestPoint)
    cSrcTranspose = numpy.transpose(cSrcPoint)
    tmp = []
    for j in range(3):
        tmp = []
        for k in range(3):
            tmp.append(numpy.dot(cSrcTranspose[j],cDestTranspose[k]))
        covariance.append(scale(1.0/float(pointNum),tmp))
    
    # apply SVD
    u,s,vh=numpy.linalg.svd(covariance)
    v=numpy.transpose(vh)
    rotation=numpy.dot(v,numpy.transpose(u))
    if numpy.linalg.det(rotation)<0:
        v[:,2]=-v[:,2]
        rotation=numpy.dot(v,numpy.transpose(u))
    
    # Compute scaling;
    scaling = sum(s)/sigma_source;
    
    # Compute translation;
    translation=avgDest-scale(scaling,numpy.dot(rotation,avgSrc));
    
    return [rotation,translation,scaling]

def EstimateSimilarityTransformationGeneral(srcPointSet,destPointSet,dimension):
    if len(srcPointSet)!=len(destPointSet):
        print 'Inconsistent sizes between two point sets!'
        return [[]]*len(destPointSet)

    if len(srcPointSet)<3:
        print 'Not enough point to estimate similarity transformation'
        return [[]]*3
    
    pointNum=len(srcPointSet)
    
    #Compute mean
    avgSrc=[0]*dimension
    avgDest=[0]*dimension
    
    for i in range(pointNum):
        avgSrc=addvec(avgSrc,srcPointSet[i])
        avgDest=addvec(avgDest,destPointSet[i])
    avgSrc=scale(1.0/float(pointNum),avgSrc)
    avgDest=scale(1.0/float(pointNum),avgDest)
    
    # compute variance of the source points
    srcTranspose = numpy.transpose(srcPointSet)
    sigma_source = 0
    for j in range(dimension):
        sigma_source += numpy.var(srcTranspose[j])
    
    # get translated source and destination points
    cSrcPoint=[[]]*pointNum
    cDestPoint=[[]]*pointNum
    for i in range(pointNum):
        cSrcPoint[i]=subvec(srcPointSet[i],avgSrc)
        cDestPoint[i]=subvec(destPointSet[i],avgDest)
    
    # compute covariance matrix between source and destination points
    covariance = []
    cDestTranspose = numpy.transpose(cDestPoint)
    cSrcTranspose = numpy.transpose(cSrcPoint)
    tmp = []
    for j in range(dimension):
        tmp = []
        for k in range(dimension):
            tmp.append(numpy.dot(cSrcTranspose[j],cDestTranspose[k]))
        covariance.append(scale(1.0/float(pointNum),tmp))
    
    # apply SVD
    u,s,vh=numpy.linalg.svd(covariance)
    v=numpy.transpose(vh)
    rotation=numpy.dot(v,numpy.transpose(u))
    if numpy.linalg.det(rotation)<0:
        v[:,dimension-1]=-v[:,dimension-1]
        rotation=numpy.dot(v,numpy.transpose(u))
    
    # Compute scaling;
    scaling = sum(s)/sigma_source;
    
    # Compute translation;
    translation=avgDest-scale(scaling,numpy.dot(rotation,avgSrc));
    
    return [rotation,translation,scaling]
    
def ApplySimilarityTransformation(srcPointSet,rotation,translation,scaling):
    destPointSet=[[]]*len(srcPointSet)
    for i in range(len(srcPointSet)):
        destPointSet[i]=addvec(translation,numpy.dot(scale(scaling,srcPointSet[i]),numpy.transpose(rotation)))
    
    return destPointSet
    

def ApplyReverseSimilarityTransformation(srcPointSet,rotation,translation,scaling):
    destPointSet=[[]]*len(srcPointSet)
    for i in range(len(srcPointSet)):
        destPointSet[i]=scale(1/float(scaling),numpy.dot(subvec(srcPointSet[i],translation),numpy.linalg.inv(numpy.transpose(rotation))))
    return destPointSet
    
def ApplyRigidTransformation(srcPointSet,rotation,translation):
    destPointSet=[[]]*len(srcPointSet)
    for i in range(len(srcPointSet)):
        destPointSet[i]=addvec(translation,numpy.dot(scale(1.0,srcPointSet[i]),numpy.transpose(rotation)))
    size = numpy.shape(destPointSet)
    destPointSet = numpy.reshape(destPointSet, (size[0], size[1]))
    return destPointSet

def ApplyReverseRigidTransformation(srcPointSet,rotation,translation):
    destPointSet=[[]]*len(srcPointSet)
    for i in range(len(srcPointSet)):
        destPointSet[i]=scale(1.0,numpy.dot(numpy.transpose(subvec(srcPointSet[i],translation)),numpy.linalg.inv(numpy.transpose(rotation))))
    size = numpy.shape(destPointSet)
    destPointSet = numpy.reshape(destPointSet, (size[0], size[2]))
    return destPointSet
    
def SimilarityTransformation(srcPointSet,destPointSet):
    [rotation,translation,scaling] = EstimateSimilarityTransformation(srcPointSet,destPointSet)
    #print 'rotation=',rotation
    #print 'translation=',translation
    #print 'scaling=',scaling
    transformedSrcPointSet=ApplySimilarityTransformation(srcPointSet,rotation,translation,scaling)
    return transformedSrcPointSet

def SimilarityTransformationGeneral(srcPointSet,destPointSet,dimension):
    [rotation,translation,scaling] = EstimateSimilarityTransformationGeneral(srcPointSet,destPointSet,dimension)
    #print 'rotation=',rotation
    #print 'translation=',translation
    #print 'scaling=',scaling
    transformedSrcPointSet=ApplySimilarityTransformation(srcPointSet,rotation,translation,scaling)
    return transformedSrcPointSet
    
def RigidTransformation(srcPointSet,destPointSet):
    [rotation,translation] = EstimateRigidTransformation(srcPointSet,destPointSet)
    #print 'rotation=',rotation
    #print 'translation=',translation
    transformedSrcPointSet=ApplyRigidTransformation(srcPointSet,rotation,translation)
    return transformedSrcPointSet
    
def RigidTransformationGeneral(srcPointSet,destPointSet,dimension):
    [rotation,translation] = EstimateRigidTransformationGeneral(srcPointSet,destPointSet,dimension)
    #print 'rotation=',rotation
    #print 'translation=',translation
    transformedSrcPointSet=ApplyRigidTransformation(srcPointSet,rotation,translation)
    return transformedSrcPointSet
    
### File format converter

def iso2vtk(iso_file,vtk_file):
    f1=open(iso_file,'r')
    print 'Start reading...'
    line=f1.readlines()
    print 'Read finished!'
    f1.close()
    
    [vertNum,triangleNum]=re.split('\t',line[0])
    vertNum=int(vertNum)
    triangleNum=int(triangleNum)
    print 'vertNum=',vertNum,'   triangleNum=',triangleNum

    print 'Start writing ... '
    f2=open(vtk_file,'w')
    string='# vtk DataFile Version 3.0\n'
    f2.write(string)    
    string='vtk output\n'
    f2.write(string)
    string='ASCII\n'
    f2.write(string)
    string='DATASET POLYDATA\n'
    f2.write(string)

    string='POINTS '+str(vertNum)+' float\n'
    f2.write(string)
    
    for i in range(1,vertNum+1):
        cord=re.split('\t',line[i])
        string=cord[0]+' '+cord[1]+' '+cord[2]
        f2.write(string)

    string='POLYGONS '+str(triangleNum)+' '+str(triangleNum*4)+'\n'
    f2.write(string)
    for i in range(vertNum+1,vertNum+triangleNum+1):
        cord=re.split('\t',line[i])
        string='3 '+cord[0]+' '+cord[1]+' '+cord[2]
        f2.write(string)
    f2.close()

    print 'Write finished!'    
    return
    
def off2vtk(off_file,vtk_file):
    f1=open(off_file,'r')
    print 'Start reading...'
    line=f1.readlines()
    print 'Read finished!'
    f1.close()
    
    [vertNum,triangleNum, dummy]=line[1].split()
    vertNum=int(vertNum)
    triangleNum=int(triangleNum)
    print 'vertNum=',vertNum,'   triangleNum=',triangleNum

    print 'Start writing ... '
    f2=open(vtk_file,'w')
    string='# vtk DataFile Version 3.0\n'
    f2.write(string)    
    string='vtk output\n'
    f2.write(string)
    string='ASCII\n'
    f2.write(string)
    string='DATASET POLYDATA\n'
    f2.write(string)

    string='POINTS '+str(vertNum)+' float\n'
    f2.write(string)
    
    for i in range(1,vertNum+1):
        cord=line[i+1].split()
        string=cord[0]+' '+cord[1]+' '+cord[2] + '\n'
        f2.write(string)

    string='POLYGONS '+str(triangleNum)+' '+str(triangleNum*4)+'\n'
    f2.write(string)
    for i in range(vertNum+1,vertNum+triangleNum+1):
        cord=line[i+1].split()
        string=cord[0]+' '+cord[1]+' '+cord[2] + ' ' +cord[3] + '\n'
        f2.write(string)
    f2.close()

    print 'Write finished!'    
    return
    
### obj file is for the display purpose (using Shaoting's visualization tool)
def iso2obj(iso_file,obj_file):
    f1 = open(iso_file, 'r')
    f2 = open(obj_file, 'w')
    size = f1.readline()
    f2.write('Obj\n')
    f2.write(size)
    size = size.split()
    
    for i in range(int(size[0])):
        cur = f1.readline()
        f2.write('v ' + cur)
        
    for i in range(int(size[1])):
        cur = f1.readline()
        f2.write('f ' + cur)
        
    print 'Write finished!'    
    return
    
def off2obj(off_file,obj_file):
    f1 = open(off_file, 'r')
    lines = f1.readlines()
    f1.close()
    f2 = open(obj_file, 'w')
    size = lines[1]
    f2.write('Obj\n')
    f2.write(size)
    size = size.split()
    
    for i in range(int(size[0])):
        cur = lines[i + 2]
        f2.write('v ' + cur)
        
    for i in range(int(size[1])):
        cur = lines[i + 2 + int(size[0])]
        cur = cur.split()
        f2.write('f\t' + str(cur[1]) + '\t' + str(cur[2]) + '\t' + str(cur[3]) + '\n')
        
    print 'Write finished!'    
    return

def off2iso(off_file, iso_file):
    off2obj(off_file, iso_file)
    obj2iso(iso_file, iso_file)
    
def obj2iso(obj_file,iso_file):
    f1 = open(obj_file, 'r')
    lines = f1.readlines()
    f1.close()
    f2 = open(iso_file, 'w')
    size = lines[1]
    print size
    size = size.split()
    f2.write(size[0] + '\t' + size[1] + '\n')
    
    for i in range(int(size[0])):
        cur = lines[i + 2]
        cur = cur.split()
        f2.write(cur[1] + '\t' + cur[2] + '\t' + cur[3] + '\n')
        
    for i in range(int(size[1])):
        cur = lines[i + 2 + int(size[0])]
        cur = cur.split()
        f2.write(cur[1] + '\t' + cur[2] + '\t' + cur[3] + '\n')
        
    print 'Write finished!'    
    return    
    
def obj2vtk(obj_file,vtk_file):
    f1=open(obj_file,'r')
    print 'Start reading...'
    line=f1.readlines()
    print 'Read finished!'
    f1.close()
    
    [vertNum,triangleNum]=re.split('\t',line[1])
    vertNum=int(vertNum)
    triangleNum=int(triangleNum)
    print 'vertNum=',vertNum,'   triangleNum=',triangleNum

    print 'Start writing ... '
    f2=open(vtk_file,'w')
    string='# vtk DataFile Version 3.0\n'
    f2.write(string)    
    string='vtk output\n'
    f2.write(string)
    string='ASCII\n'
    f2.write(string)
    string='DATASET POLYDATA\n'
    f2.write(string)

    string='POINTS '+str(vertNum)+' float\n'
    f2.write(string)
    
    for i in range(2,vertNum+2):
        cord = (line[i]).split()
        string=cord[1]+' '+cord[2]+' '+cord[3] + '\n'
        f2.write(string)

    string='POLYGONS '+str(triangleNum)+' '+str(triangleNum*4)+'\n'
    f2.write(string)
    for i in range(vertNum+2,vertNum+triangleNum+2):
        cord = (line[i]).split()
        string='3 '+cord[1]+' '+cord[2]+' '+cord[3] + '\n'
        f2.write(string)
    f2.close()

    print 'Write finished!'    
    return    
    

def vtk2iso(vtk_file,iso_file):
    f1=open(vtk_file,'r')
    print 'Start reading...'
    line=f1.readlines()
    print 'Read finished!'
    f1.close()

    if line[0].find('vtk')==-1:
        print 'Illegal input file!'
        return

    #Look for vertex number
    vertLine=0
    foundVertex=False
    for lineIndex in range(len(line)):
        if line[lineIndex].find('POINTS')>=0:
            words=re.split(' ',line[lineIndex])
            vertNum=int(words[1])
            vertLine=lineIndex+1
            foundVertex=True
            break

    if foundVertex==False:
        print 'Can not find vertex information!'
        return

    triangleLine=0
    foundTriangle=False
    for lineIndex in range(vertLine,len(line)):
        if line[lineIndex].find('POLYGONS')>=0:
            words=re.split(' ',line[lineIndex])
            triangleNum=int(words[1])
            triangleLine=lineIndex+1
            foundTriangle=True
            break

    if foundTriangle==False:
        print 'Can not find triangle information!'
        return

    normalLine=0
    foundNormal=False
    for lineIndex in range(triangleLine,len(line)):
        if line[lineIndex].find('NORMALS')>=0:
            words=re.split(' ',line[lineIndex-1])
            normNum=int(words[1])
            normalLine=lineIndex+1
            foundNormal=True
            break

    #if foundNormal==False or normNum!=vertNum:
    #    print 'Can not find normal information!'
    #    return

    print 'Start writing ... '
    f2=open(iso_file,'w')

    string=str(vertNum)+'\t'+str(triangleNum)+'\n'
    f2.write(string)

    remain= vertNum % 3
    if remain == 0:
        totalLine=int(vertNum/3)
    else:
        totalLine=int(vertNum/3)+1
            
    for lineIndex in range(totalLine):
        vertCord=re.split(' ',line[lineIndex+vertLine])
        normalCord=re.split(' ',line[lineIndex+normalLine])
        if remain==0 or lineIndex!=totalLine-1:            
            for i in range(3):                
                string=vertCord[i*3]+'\t'+vertCord[i*3+1]+'\t'+vertCord[i*3+2]+'\n'#+normalCord[i*3]+'\t'+normalCord[i*3+1]+'\t'+normalCord[i*3+2]+'\n'
                f2.write(string)
        else:
            for i in range(remain):                
                string=vertCord[i*3]+'\t'+vertCord[i*3+1]+'\t'+vertCord[i*3+2]+'\n'#+normalCord[i*3]+'\t'+normalCord[i*3+1]+'\t'+normalCord[i*3+2]+'\n'
                f2.write(string)

    for lineIndex in range(triangleNum):
        triangleInfo=re.split(' ',line[lineIndex+triangleLine])
        string=triangleInfo[1]+'\t'+triangleInfo[2]+'\t'+triangleInfo[3]+'\n'
        f2.write(string)
    
    f2.close()
    print 'Write finished!'    
    
    return

def vtk2off(vtk_file,off_file):
    vtk2iso(vtk_file, off_file)
    iso2off(off_file, off_file)

def iso2off(iso_file,off_file):
    f1=open(iso_file,'r')
    print 'Start reading...'
    line=f1.readlines()
    print 'Read finished!'
    f1.close()

    
    [vertNum,triangleNum]=(line[0]).split()
    vertNum=int(vertNum)
    triangleNum=int(triangleNum)
    print 'vertNum=',vertNum,'   triangleNum=',triangleNum
    
    print 'Start writing ... '
    f2=open(off_file,'w')

    string='OFF\n'
    f2.write(string)

    string=str(vertNum)+'\t'+str(triangleNum)+'\t'+'0\n'
    f2.write(string)

    for i in range(1,vertNum+1):
        cord=(line[i]).split()
        string=cord[0]+'\t'+cord[1]+'\t'+cord[2]
        if string[len(string)-1]!='\n':
            string +='\n'
        f2.write(string)

    for i in range(vertNum+1,vertNum+triangleNum+1):
        cord=(line[i]).split()
        string='3\t'+cord[0]+'\t'+cord[1]+'\t'+cord[2] + '\n'
        f2.write(string)
    
    f2.close()
    print 'Write finished!'        
    return
